# Master Configure file for the INL 4.0 server setup
# also used in the BRM client
# 

# What architecture is this for?
#
# Possible values (add to these as necessary):
#    Solaris
#    SYSV
#    ultrix
#    HPBSD
#    Linux
#    AIX32
#    sun4
#    sgi

ARCH = ultrix


# NOTE: many systems' /bin/cc may choke on much of the code in ntserv/ use with
# caution!
CC= cc

# Some system, particular Sys V and sgi, may need to define SHELL
#SHELL = /bin/sh

#       Directories:
#          BINDIR    - where the binaries will end up
#          LIBDIR     - where the lib files will go (.sysdef, .motd, etc)
#          XTREKDIR   - root src dir
#          XTREKOWNER - Owner of the game
# July 6 1993 - hasn't been tested for BINDIR, LIBDIR != XTREKDIR

XTREKDIR = /volta/users/petonak/bin/INL/bin.alpha
BINDIR = $(XTREKDIR)
LIBDIR = $(XTREKDIR)
XTREKOWNER = petonak


#       Some systems don't have strdup()
#       Comment out if you have strdup()

#STRDUPC	= strdup.c
#STRDUPO	= strdup.o

#       Some systems dont have install so you may need to use cp instead

INSTALL		= cp
INSTALLOPTS	= 

# This should point to the _gnu_ indent program (v1.8 or better)
#INDENT		= /usr/local/gnu/bin/indent


#       Either use gmp or mp - depends on your system.
#       Comment out if your system has mp and define them below these...
# Directory to find multiprecision library and includes.
# if you  don't have libmp.a you are probably hosed .. but it's easy to
# get gmp from the GNU people.
#
# Solaris and most BSD based machines probably needs:
#    MPLIB      = (not required)
#    MPINC      = (not required)
#    LMP        = -lmp
#
# SYSV needs:
#    MPLIB      = (set to correct lib location)
#    MPINC      = (set to correct include location)
#    LMP        = -lmp -lgmp -lPW
#
# My MP dir    (Set if you don't have gmp installed in standard location)
#    GMP        = /user/trown/gmp-1.2
#    MPLIB      = -L$(GMP)
#    MPINC      = -I$(GMP)
#    LMP        = -lmp -lgmp
#
#       For systems that have mp
#MPLIB = -lmp
#MPINC =

GMP		= /volta/users/petonak/bin/INL/gmp-1.3.2
MPLIBRARY	= -L$(GMP)
MPINCLUDE	= -I$(GMP)
LMP		= -lgmp -lmp

# server only
MPLIB		= $(MPLIBRARY) $(LMP)


# Directory to find X11 libs includes, and link options
#
# Solaris and SYSV probably needs:
#    X11LIB     = (not required)
#    X11INC     = (not required)
#    LX         = -lX11
# xsg and BRM use only
# not sure if xsg is hosed by this... - jn

X11INC		= -I/usr/include/X11
X11LIB		= -L/usr/lib
LX		= -lX11

#       Optimizer flags:
#          +O3  - Highest optimization
#          -O     Normal optimization (+O2)
#          -g     Debugging

OPTS =  -g

# RSA stuff
# -DRSA - rsa support thanks to ebrown@ecn.purdue.edu and sam shen
RSA_FLAGS = -DRSA
RSA_INCS = $(MPINC)
RSA_LIBS = $(MPLIB)

# for the server only, see below for client RSA files
RSA_OBJS = rsa_key.o rsa_utilmp.o
RSA_SRC = rsa_key.c rsa_utilmp.c


#  Customizations for individual platforms should go here...
#  useful flags:
# -D EXIT_DEFINED  (for compilers that choke in ntserv/socket.c 
# 			on the int exit()  definition).
# -DERIKPLAGUE - erik lauer style plague code
# -DSTDBG - enables various debugging output messages
# -DSHORT_PACKETS - enables 'short' packets by Tedd Hadley and Heiko Wengler
# 		    Bug reports to hw@plato.Chemietechnik.Uni-Dortmund.DE
# -DSHORT_THRESHOLD - (experimental) lets people set max CPS threshhold
# 	current rumour is .. avoid it..
# -DNOTIMEB - replace ftime() with gettimeofday() by siegl@risc.uni-linz.ac.at
# 	    - needed for sgi 
# -DNEWSHIP - use altered (non-INL legal) ship specs (check getship.c)
#
# -DDS  - have daemon control updates of server processes (safer for ghostbusts,
#         controlled slot frees, smoother updates, etc.) SLOT_PID is not needed
#         if -DDS is used.
# -DEXTRACREDIT - give bonus for core worlds and such
# -DNOCLWARN - dont warn people if their client is old
# -DBD  - borg detection
# for SysV/H-Puke

EXTRACFLAGS = -DPING -DDS $(RSA_FLAGS) -DERIKPLAGUE \
	-DSHORT_PACKETS -DEXTRACREDIT -DEXIT_DEFINED -DNOTIMEB -DBD

#
# SYSV needs -lV3 (PW is set on LGMP line)
#    OLIB       = -lV3
#    OINC       = (not required)
#    LO         = -lm
#
#
# Solaris probably needs:
#    OLIB       = (not required)
#    OINC       = (not required)
#    LO         = -lnsl -lsocket -lm
#
# ultrix probably needs:
#    OLIB       = (not required)
#    OINC       = (not required)
#    LO         = -lm

#OLIB		=
#OINC		=
LO		= -lm

#server only
EXTRALIBS = $(LO) $(X11LIB) $(LX)


#       Some systems need X11R5 in the include path
#       Comment out if you don't need this

EXTRAINCS = $(MPINCLUDE)





