# Master Configure file for the INL 4.0 server setup
# also used in the BRM client
# 

# What architecture is this for?
#
# Possible values (add to these as necessary):
#    Solaris
#    SYSV
#    ultrix
#    HPBSD
#    Linux
#    AIX32
#    sun4
#    sgi

ARCH = sgi


# NOTE: many systems' /bin/cc may choke on much of the code in ntserv/ use with
# caution!
CC= cc -cckr

# Some system, particular Sys V and sgi, may need to define SHELL
SHELL = /bin/sh

#       Directories:
#          BINDIR    - where the binaries will end up
#          LIBDIR     - where the lib files will go (.sysdef, .motd, etc)
#          XTREKDIR   - root src dir
#          XTREKOWNER - Owner of the game
# July 6 1993 - hasn't been tested for BINDIR, LIBDIR != XTREKDIR

XTREKDIR = /volta/users/petonak/bin/INL/bin.sgi
BINDIR = $(XTREKDIR)
LIBDIR = $(XTREKDIR)
XTREKOWNER = sgi


#       Some systems don't have strdup()
#       Comment out if you have strdup()

#STRDUPC = strdup.c
#STRDUPO = strdup.o

#       Some systems dont have install so you may need to use cp instead

INSTALL = -cp
INSTALLOPTS =


#       Either use gmp or mp - depends on your system.
#       Comment out if your system has mp and define them below these...
# Directory to find multiprecision library and includes.
# if you  don't have libmp.a you are probably hosed .. but it's easy to
# get gmp from the GNU people.
#
# Solaris and most BSD based machines probably needs:
#    MPLIB      = (not required)
#    MPINC      = (not required)
#    LMP        = -lmp
#
# SYSV needs:
#    MPLIB      = (set to correct lib location)
#    MPINC      = (set to correct include location)
#    LMP        = -lmp -lgmp -lPW
#
# My MP dir    (Set if you don't have gmp installed in standard location)
#    GMP        = /user/trown/gmp-1.2
#    MPLIB      = -L$(GMP)
#    MPINC      = -I$(GMP)
#    LMP        = -lmp -lgmp
#
#       For systems that have mp
#MPLIB = -lmp
#MPINC =

GMP		= /volta/users/petonak/bin/INL/gmp-1.3.2
MPLIBRARY	= -L$(GMP)
MPINCLUDE	= -I/volta/users/petonak/bin/INL/gmp-1.3.2
LMP		= -lgmp -lmp

# server only
MPLIB		= $(MPLIBRARY) $(LMP)


# Directory to find X11 libs includes, and link options
#
# Solaris and SYSV probably needs:
#    X11LIB     = (not required)
#    X11INC     = (not required)
#    LX         = -lX11
# xsg and BRM use only
# not sure if xsg is hosed by this... - jn

X11INC		= -I/usr/X11/include
X11LIB		= -L/usr/X11/lib
LX		= -lX11

#       Optimizer flags:
#          +O3  - Highest optimization
#          -O     Normal optimization (+O2)
#          -g     Debugging

OPTS =  -g


# RSA stuff
# -DRSA - rsa support thanks to ebrown@ecn.purdue.edu and sam shen
RSA_FLAGS = -DRSA
RSA_INCS = $(MPINC)
RSA_LIBS = $(MPLIB)

# for the server only, see below for client RSA files
RSA_OBJS = rsa_key.o rsa_utilmp.o
RSA_SRC = rsa_key.c rsa_utilmp.c


#  Customizations for individual platforms should go here...
#  useful flags:
# -D EXIT_DEFINED  (for compilers that choke in ntserv/socket.c 
# 			on the int exit()  definition).
# -DERIKPLAGUE - erik lauer style plague code
# -DSTDBG - enables various debugging output messages
# -DSHORT_PACKETS - enables 'short' packets by Tedd Hadley and Heiko Wengler
# 		    Bug reports to hw@plato.Chemietechnik.Uni-Dortmund.DE
# -DSHORT_THRESHOLD - (experimental) lets people set max CPS threshhold
# 	current rumour is .. avoid it..
# -DNOTIMEB - replace ftime() with gettimeofday() by siegl@risc.uni-linz.ac.at
# 	    - needed for sgi 
# -DNEWSHIP - use altered (non-INL legal) ship specs (check getship.c)
#
# -DDS  - have daemon control updates of server processes (safer for ghostbusts,
#         controlled slot frees, smoother updates, etc.) SLOT_PID is not needed
#         if -DDS is used.
#
# -DEXTRACREDIT - give bonus for core worlds and such
# -DBD  - borg detection
# for SysV/H-Puke

EXTRACFLAGS = -DPING -DDS $(RSA_FLAGS) -DERIKPLAGUE -DSHORT_PACKETS \
	-DNOTIMEB -DSYSV -DDTEST -DEXTRACREDIT -DBD


# Other libs and includes, all will need -lm
#
# SYSV needs -lV3 (PW is set on LGMP line)
#    OLIB       = -lV3
#    OINC       = (not required)
#    LO         = -lm
#
#
# Solaris probably needs:
#    OLIB       = (not required)
#    OINC       = (not required)
#    LO         = -lnsl -lsocket -lm
#
# ultrix probably needs:
#    OLIB       = (not required)
#    OINC       = (not required)
#    LO         = -lm

#OLIB		=
#OINC		=
LO		= -lm

#server only
EXTRALIBS = $(LO) $(X11LIB) $(LX)


#       Some systems need X11R5 in the include path
#       Comment out if you don't need this

EXTRAINCS = $(MPINCLUDE)







# BRM stuff
# these define effect only BRM compilation, do not worry about them
# if you are compiling a server


# Keyfile.  set the name of the file containing the secret key for mkkey
# to use.

KEYFILE         = ../berkrick-nelson-sgi.secret

# NOTE: If you have to make a key from keys.h then after you have changed
# the following 5 params do:
#       make convert

KEYSH           = keys.h
KEYNAME         = $(ARCH)
DESC            = "sun4 / sunos4.1"
MAKER           = lab@mtek.chalmers.se
COMMENT         = "reserved.c blessed. Located at ftp.cd.chalmers.se"

# don't define these if you don't have RSA
RSASRC          = rsa-client.c rsa_clientutil.c
RSAOBJ          = rsa-client.o rsa_clientutil.o


# in the future, maybe the server will support some of these options
# but for not they are still BRM only

# Link options
#   gcc requires -static to link statically
#   Sun cc requires -Bstatic
#   HPUX cc requires -Wl,-a,archive -s
#   HPBSD only needs -s
#
#   -g for debugging: usually causes it to be statically linked

LOPTS           = $(OPTS)

# random.c  Some arches don't have random(), we will compile it if necessary
# set RANDOMOBJ = random.o if you need this, otherwise leave it blank.
# for this to work you should also NORDEFINE = -DNORANDOM
#       -DNORANDOM - your libc is braindamaged (ie. you dont have a random()!)
#
# Solaris and SYSV probably need:
#   RANDOMOBJ   = random.o
#   NORDEFINE     = -DNORANDOM


# RANDOMOBJ     =
# NORDEFINE     =

# Defines
#    Some possible values:
#
#       -DNEED_EXIT
# some systems need exit() defined in socket.c, some don't
#
#       -DRSA - New RSA reserved packets stuff
#       -DNBT - Nick trown's macro code
#       -DNBTDIST - Nick trown's intelligent distress call code
#       -DNBTDISTVAR - let .xtrekrc define distress calls
#       -DMAXPLAYER=36
#       -DDEBUG    Various useful debugging stuff.  No signal trapping
#       -DCORRUPTED_PACKETS - prevents some seg faults but verbose output
#       -D_IBMR2 - weird IBM RS6000 AIX3.2  system support
#       -DSHORT_PACKETS - implement short_packets
#       -DEXPIRE=#      - number of days this version will work for
#       -DARMY_SLIDER   - some sort of funky status window thing
#       -DMETA          - show active servers via metaserver - NBT
#       -DPACKET_LOG    - log packets to stdout (for bandwith measurement)
#       -DNEWMACRO - newmacro language
#       -DSMARTMACRO - macro extension (needs NEWMACRO)


DEFINES         = -D$(ARCH) $(NORDEFINE) -DNBT -DNBTDIST -DCORRUPTED_PACKETS -DPACKET_LOG\
		-DSHORT_PACKETS -DEXPIRE=30 -DARMY_SLIDER -DMETA\
		-DNEWMACRO -DSMARTMACRO -BD


# How to generate cwho in cflags.h.  Some hosts fully resolve the name
# others need to tack on the domainname.
# you server hacks probably want this also... - jn
#
# three possitilities:
# CWHO          = \"`whoami`@`hostname`\"
# CWHO          = \"`whoami`@`domainname`\"
# CWHO          = \"`whoami`@`hostname`.`domainname`\"

CWHO            = \"`whoami`@`hostname`\"


# end BRM stuff

# xsg stuff
X11INCLUDE = $(EXTRAINCS) $(X11INC)


# end xsg stuff
