



/* 
 * findslot.c
 *
 * Kevin Smith 03/23/88
 *
 * almost entirely rewritten
 * Eric mehlhaff, 7/24/92
 *
 * almost entirely rewritten
 * Steve Sheldon, 4/16/93     sheldon@iastate.edu
 */
#include "copyright2.h"

#include <stdio.h>
#include <sys/types.h>
#include <errno.h>
#include <pwd.h>
#include <string.h>
#include <ctype.h>
#include "defs.h"
#include "struct.h"
#include "data.h"
#include <signal.h>

int debug = 0;

findslot (overload, queue_num)
     int overload;
{
  int i;
  int j;

  i = newgrabslot (overload, queue_num);
  players[i].p_pos = -1;
  bzero (&players[i].p_stats, sizeof (struct stats));
  if (status->tourn && players[i].p_stats.st_tticks == 0)
    {
      players[i].p_stats.st_tticks = 1;
    }
  for (j = 0; j < 95; j++)
    {
      players[i].p_stats.st_keymap[j] = j + 32;
    }
  players[i].p_stats.st_keymap[95] = 0;
  players[i].p_stats.st_flags = ST_INITIAL;
  return (i);
}

/*
 * This is out of date, I'll explain it better later... SRS 4/16/93
 *
 * The following code for newgrabslot() is much different... 
 *  Here's an explaination:
 *
 * The queue list is an array of ints (client process pid's)
 * It's kept in order. Periodically, a process will get let in, and
 * it will clean up anf re-order the queue list.
 * all waiting processes will continually update a timestamp, showing
 * when the last time they 'ran'. This helps let them tell if one
 * is messed up and should be dropped from the list
 *
 * When there's a double-queue, the second queue works the same
 * as the first. It's array index is in the main queue list array
 * but it is half-way out.
 *
 * Periodically, processes will check the queue to make sure it hasn't
 * been corrupted.
 *
 * Every process will wait for a second (sleep(1)), and then check the queue
 *   status, do what is appropriate, and repeat.
 *
 * Above and beyond this, processes may die, and not report it (for whatever
 *   reason, and every process waiting on the queue watches the behavior of
 *   every other supposed process to make sure it is still alive).
 *   This is accomplished via kill(pid, 0) signals.
 *
 * Variables:
 *   wait_queue->Queue_List[MAXQUEUE] the queue list. 
 *             ->Queue_TimeStamps[MAXQUEUE] last time process updated itself
 *             ->Second_Queue   Index number of first entry in second
 *                      wait queue, if 0, there is no second queue.
 *             ->lock   Whether the list is locked, so other processes may
 *                      much about with it while one is busy
 *                      0 == not locked
 *                      !0  locked by pid stored in this variable
 *             ->lock_time when it was locked.  Get worried if this is
 *                       too far from now.
 *
 * NOTE: UNLIKE THE OLD GRABSLOT(), THIS _WILL_ BREAK IF THE WHOLE NETREK
 * SERVER  
 * GETS PUT INTO A SINGLE PROCESS. HAVE A NICE DAY.
 */

#ifndef PATIENCE
/* number of seconds we will wait for queue to unlock
   ** or for the first person on the queue_list to try entering the game
 */
#define PATIENCE  (time_t) 10
#endif

#ifndef INL_TEAMAX
#define INL_TEAMAX 8
#endif

int
newgrabslot (overload, queue_num)
     int overload;		/* Indicates a request for a tester's slot. */
     int queue_num;		/* which queue to try to be on. 0 or 1 */
{
  int i;
  int myPid = 0;
  int wearein = -1;		/* Slot we got in the game */
  int try;
  int nump;
  int myqueuepos = -1;
  int lastqueuepos = -1;
  int updatecount = 0;

  myPid = getpid ();

  if (debug)
    {
      printf ("Finding a slot for player!\n");
      fflush (stdout);
    }

  while (wearein < 0)
    {

      /* a 'clock' for the updates */
      updatecount++;

      /* check sysdefaults */
      readsysdefaults ();

      /* get queue position, show that we're alive */
      myqueuepos = Queue_Position (myPid);
      if (myqueuepos >= 0)
	{
	  if (queue_num == 1)
	    {
	      wait_queue->Queue_TimeStamps[myqueuepos +
				    wait_queue->Second_Queue] = time (NULL);
	    }
	  else
	    {
	      wait_queue->Queue_TimeStamps[myqueuepos] = time (NULL);
	    }
	}
      if (debug > 1)
	{
	  printf ("Waitqueue update %d: position %d \n", updatecount,
		  myqueuepos);
	  fflush (stdout);
	}


      /* are we on the queue already? if not, try add us!! */
      if (myqueuepos < 0)
	{
	  /* try to get on */
	  if (Add_To_Queue (myPid, queue_num + 1) == 0)
	    {
	      /* can't get on?  arggh. */
	      if (updatecount % 10 == 0)
		{
		  mapWaitCount (6666 + myqueuepos);
		}
	    }
	  else
	    {
	      /* a queue postition has been grabbed */
	      myqueuepos = Queue_Position (myPid);
	    }
	}

      /* timeout overly long lock on wait queue */
      if (wait_queue->lock != 0 &&
	  ABS (time (NULL) - wait_queue->lock_time) > PATIENCE)
	{
	  UnlockQueue ();
	  continue;
	}


      /* attempt to enter if first on queue */
      if ((overload || myqueuepos == 0) &&
	  (wearein = AttemptEntry (myPid, queue_num, overload)) >= 0)
	continue;


      /* We had been checking the entire queue to see if it was
         corrupt.  I figure we should just check the guy in front of
         us, and make sure he's alive */
      if (CheckQueue (queue_num + 1, myPid) == 1)
	myqueuepos = Queue_Position (myPid);

      /* map wait queue position if it has changed */
      if (lastqueuepos != myqueuepos || (updatecount % 10 == 0))
	{
	  lastqueuepos = myqueuepos;
	  mapWaitCount ((myqueuepos >= 0) ? myqueuepos : 666);

	  /* break out ahead of the sleep so that the queue may move
	     ** faster when it actually is moving! */
	  continue;
	}


      sleep (1);
    }
  return (wearein);
}



AttemptEntry (myPid, queue_num, overload)
     int myPid;
     int queue_num;
     int overload;
{
  int wegotin = -1;
  int gotlock = 0;
  int mypos = -1;

  while (gotlock == 0)
    {
      while (wait_queue->lock)
	{
	  sleep (1);
	  if (time (NULL) > wait_queue->lock_time + (time_t) PATIENCE)
	    UnlockQueue ();
	}

      wait_queue->lock_time = time (NULL);
      wait_queue->lock = getpid ();

      /* Just in case something happens and another guy grabbed the queue before us */
      if (wait_queue->lock == getpid ())
	gotlock = 1;
    }

  /* Check where we are in the queue, if we aren't in there, we got trouble */
  mypos = Queue_Position (myPid);
  if (mypos < 0)
    return (-1);

  if (debug > 1)
    {
      printf ("Seeing if I can enter...");
      fflush (stdout);
    }

  /* using the normal queue */
  if (queue_num == 0 || wait_queue->Second_Queue == 0)
    {
      /* using normal queue, see if we can get in */

      /* in an INL game, the '0' queue is used for the HOME team
         ** if there's more than INL_TEAMAX in the game already on that 
         ** team, then any more are reserver players waiting to get in
       */
      if (wait_queue->Second_Queue > 0 &&
	  overload == 0 &&
	  realNumShips (INLteams[0]) >= INL_TEAMAX)
	{
	  /* home team uses normal queue.  If more than
	     ** INL_TEAMAX players on hometeam, they have to wait. */
	  if (debug > 1)
	    {
	      printf ("Nope, HomeTeam(%d) full!\n", INLteams[0]);
	      fflush (stdout);
	    }
	  wait_queue->lock = 0;
	  return (-1);
	}
    }
  else
    {				/* queue_num == 1 && Second_Queue !=0 */
      if (overload == 0 &&
	  realNumShips (INLteams[1]) >= INL_TEAMAX)
	{
	  /* visiting team is full, the rest must wait */
	  if (debug > 1)
	    {
	      printf ("Nope, VisitingTeam(%d) Full\n", INLteams[1]);
	      fflush (stdout);
	    }
	  wait_queue->lock = 0;
	  return (-1);
	}
    }


  if (debug)
    printf ("maybe...");

  /* at this point, this player is permitted to attempt to grab a slot */
  if ((wegotin = grabPlayerSlot (overload)) >= 0)
    {
      wait_queue->Queue_TimeStamps[mypos] = 0;
      wait_queue->Queue_List[mypos] = 0;
      sleep (1);
      mapWaitCount (998);
    }

  if (debug)
    {
      printf ("Entry attempt result %d\n", wegotin);
      fflush (stdout);
    }

  wait_queue->lock = 0;
  return (wegotin);
}


mapWaitCount (count)
     unsigned int count;
{
  if (count == -1)
    return;

  sendQueuePacket ((short) count);
  flushSockBuf ();
}

/* returns slot grabbed, if there were any
   ** returns -1 if there weren't
 */
int
grabPlayerSlot (overload)
     int overload;
{
  int i;
  if (overload > 0)
    {
      /* grab slot from the top down */
      for (i = MAXPLAYER - 1; i >= 0; i--)
	{
	  if (players[i].p_status == PFREE)
	    {			/* We have a free slot */

	      players[i].p_ghostbuster = 0;
	      /* make sure player wasn't at war or refitting during ghostbust
	         or our client will freeze */
	      players[i].p_flags = 0;
#ifdef DS
	      players[i].p_process = 0;
#endif

	      players[i].p_status = POUTFIT;	/* possible race code */
	      players[i].p_team = NOBODY;
	      return (i);
	    }
	}
    }
  else
    {
      for (i = 0; i < MAXPLAYER; i++)
	{
	  if (players[i].p_status == PFREE)
	    {			/* We have a free slot */

	      players[i].p_ghostbuster = 0;
	      /* make sure player wasn't at war or refitting during ghostbust
	         or our client will freeze */
	      players[i].p_flags = 0;
#ifdef DS
	      players[i].p_process = 0;
#endif

	      players[i].p_status = POUTFIT;	/* possible race code */
	      players[i].p_team = NOBODY;
	      return (i);
	    }
	}
    }
  /* not slots open. Sigh, oh well... */
  return (-1);
}

/* examine wait queue list and return the number of people on the
   ** specified wait queue. 0 == total, 1 == queue 1, 2 == queue 2
 */
int
Queue_Length (queue_num)
     int queue_num;
{
  int i;
  int numfound = 0;
  int startsearch;
  int endsearch;

  switch (queue_num)
    {
    case 1:
      startsearch = 0;
      endsearch = (wait_queue->Second_Queue == 0) ? MAXQUEUE :
	wait_queue->Second_Queue;
      break;
    case 2:
      if (wait_queue->Second_Queue == 0)
	return (MAXQUEUE + 1);
      startsearch = wait_queue->Second_Queue;
      endsearch = MAXQUEUE;
      break;
    case 0:
      startsearch = 0;
      endsearch = MAXQUEUE;
      break;
    default:
      return (MAXQUEUE + 1);
    }
  for (i = startsearch; i < endsearch; i++)
    {
      if ((wait_queue->Queue_List[i] != 0) &&
	  (time (NULL) - wait_queue->lock_time) < PATIENCE)
	numfound++;
    }
  return (numfound);
}


/* return position of process in argument in the queues */
int
Queue_Position (proc_num)
     int proc_num;
{
  int i;

  for (i = 0; i < MAXQUEUE; i++)
    {
      if (wait_queue->Queue_List[i] == proc_num)
	{
	  if (i < wait_queue->Second_Queue)
	    return (i);
	  else
	    return (i - wait_queue->Second_Queue);
	}
    }
  return (-1);
}



/* try to add person to wait queue
   ** if unable return 0
   ** otherwise return 1
 */
int
Add_To_Queue (adpid, queue_num)
     int adpid;
     int queue_num;
{
  int i;
  int startsearch;
  int endsearch;
  int gotlock = 0;
  time_t due_time;

  while (gotlock == 0)
    {
      while (wait_queue->lock)
	{
	  mapWaitCount (600);
	  sleep (1);
	  if (time (NULL) > wait_queue->lock_time + (time_t) PATIENCE)
	    UnlockQueue ();
	}

      wait_queue->lock_time = time (NULL);
      wait_queue->lock = getpid ();

/* Just in case something happens and another guy grabbed the queue before us */
      if (wait_queue->lock == getpid ())
	gotlock = 1;
    }

  switch (queue_num)
    {
    case 1:
      startsearch = 0;
      endsearch = (wait_queue->Second_Queue == 0) ? MAXQUEUE :
	wait_queue->Second_Queue;

      break;
    case 2:
      startsearch = wait_queue->Second_Queue;
      endsearch = MAXQUEUE;
      break;
    default:
    case 0:
      startsearch = 0;
      endsearch = MAXQUEUE;
      break;
    }

  due_time = time (NULL);

  for (i = startsearch; i < endsearch; i++)
    {
      if (wait_queue->Queue_List[i] == 0 || ABS (wait_queue->Queue_TimeStamps[i] - due_time) > PATIENCE)
	{
	  wait_queue->Queue_List[i] = adpid;
	  wait_queue->Queue_TimeStamps[i] = time (NULL);
	  wait_queue->lock = 0;
	  return (1);
	}
    }
  wait_queue->lock = 0;
  return (0);
}

/* return number of players (non-robot) in the game */
int
Num_Players (teammask)
     int teammask;
{
  int i;
  int numfound = 0;

  for (i = 0; i < MAXPLAYER; i++)
    {
      if (players[i].p_status == PFREE)
	continue;
      if (players[i].p_flags & PFROBOT)
	continue;
      if (players[i].p_team | teammask)
	numfound++;
    }
  return (numfound);
}

/* for some reason, some process has the queue locked for an unusually
   ** long time
 */
UnlockQueue ()
{
  int locker;

  /* debugging output! */
  printf ("Unlocking wait queue!\n");
  fflush (stdout);


  /* find out what happened to the original locking process */
  locker = wait_queue->lock;
  fprintf (stderr, "Queue locked for an excessive time, unlocking!\n");
/* There was a SIGCONT here.  I figure if a process has locked the queue for
   too long, it's probably dead.  Let's send it an interrupt and find out */
  if (kill (locker, SIGINT) < 0)
    {
      fprintf (stderr, "By pid %d", locker);
      perror ("Kill error:");
    }

  /* this process will now take over the lock */
  wait_queue->lock_time = time (NULL);
  wait_queue->lock = getpid ();

  wait_queue->lock = 0;


}

/* CheckQueue()
   /* The new and improved CheckQueue()
   /* This procedure checks the guy in front of you.  If his timestamp is too
   /* old, or he's no longer around, we move up a slot!
 */

int
CheckQueue (queue_num, myPid)
     int queue_num;
     int myPid;
{
  int i;
  int startsearch;
  int endsearch;
  time_t due_time;
  int gotlock = 0;
  int *Qlist = wait_queue->Queue_List;
  time_t *TimeStamp = wait_queue->Queue_TimeStamps;
  int mypos;

  due_time = time (NULL);

  switch (queue_num)
    {
    case 1:
      startsearch = 0;
      endsearch = (wait_queue->Second_Queue == 0) ? MAXQUEUE :
	wait_queue->Second_Queue;
      break;
    case 2:
      /* if (wait_queue->Second_Queue == 0) return(MAXQUEUE+1); */
      /* clean full queue if split queue is off... */
      startsearch = wait_queue->Second_Queue;
      endsearch = MAXQUEUE;
      break;
    case 0:
    default:
      startsearch = 0;
      endsearch = MAXQUEUE;
    }

/* We keep start and end.  startsearch is the first position in our
   /* queue.  If we're at that position, who really cares, right?
 */
  /* Check where we are in the queue, if we aren't in there, we got trouble */
  mypos = Queue_Position (myPid);
  if (mypos < 0)
    return (0);


  if ((mypos - 1) >= 0)
    {
      /* Well we're not at the beginning of the queue, let's check our neighbor
         /* in front of us.
       */

      /* First, let's try to lock the queue, as we're going to be modifying it
       */

      while (gotlock == 0)
	{
	  while (wait_queue->lock)
	    {
	      sleep (1);
	      if (time (NULL) > (wait_queue->lock_time + (time_t) PATIENCE))
		UnlockQueue ();
	    }

	  wait_queue->lock_time = time (NULL);
	  wait_queue->lock = getpid ();

	  /* Just in case something happens and another guy grabbed the queue before us
	   */
	  if (wait_queue->lock == getpid ())
	    gotlock = 1;

	}

      i = mypos + startsearch - 1;

      if (Qlist[i] == 0 || ABS (TimeStamp[i] - due_time) > PATIENCE)
	{
	  TimeStamp[i] = TimeStamp[mypos + startsearch];
	  Qlist[i] = Qlist[mypos + startsearch];
	  TimeStamp[mypos + startsearch] = 0;
	  Qlist[mypos + startsearch] = 0;
	  wait_queue->lock = 0;
	  return (1);
	}
      wait_queue->lock = 0;
      return (0);
    }

  return (0);
}
