/*
 * plasma.c
 */
#include "copyright.h"

#include <stdio.h>
#include <sys/types.h>
#include "defs.h"
#include "struct.h"
#include "data.h"

/* Launch torp */

/*
   ** If a set of given conditions are met, fire a single torp in direction
   ** course.  Type is used because robots are allowed to shoot straight.
   ** Torps sent with status TMOVE wobble a bit.  TSTRAIGHT torps (fired
   ** by robots) move in a straight line.
   **
   ** torp->t_fuse is the life of the torpedo.  It is set here based on
   ** a random function.  Torps currently live two to five seconds.
 */

nplasmatorp (course, type)
     unsigned char course;
     int type;
{
  register int i;
  register struct plasmatorp *k;

  if (weaponsallowed[WP_PLASMA] == 0)
    {
#ifdef SHORT_PACKETS
      warning ("Plasmas haven't been invented yet.", 17);
#else
      warning ("Plasmas haven't been invented yet.");
#endif
      return;
    }
  if (me->p_ship.s_plasmacost < 0)
    {
#ifdef SHORT_PACKETS
      spwarning ("Weapon's Officer:  Captain, this ship is not armed with plasma torpedoes!", 18);
#else
      warning ("Weapon's Officer:  Captain, this ship is not armed with plasma torpedoes!");
#endif
      return;
    }
  if (me->p_flags & PFWEP)
    {
#ifdef SHORT_PACKETS
      spwarning ("Plasma torpedo launch tube has exceeded the maximum safe temperature!", 19);
#else
      warning ("Plasma torpedo launch tube has exceeded the maximum safe temperature!");
#endif
      return;
    }

  if ((me->p_nplasmatorp == MAXPLASMA) && (me->p_ship.s_type != ATT))
    {
#ifdef SHORT_PACKETS
      spwarning ("Our fire control system limits us to 1 live torpedo at a time captain!", 20);
#else
      warning ("Our fire control system limits us to 1 live torpedo at a time captain!");
#endif
      return;
    }
  else if (me->p_nplasmatorp == MAXPLASMA)
    {
#ifdef SHORT_PACKETS
      spwarning ("Our fire control system limits us to 1 live torpedo at a time captain!", 20);
#else
      warning ("Our fire control system limits us to 1 live torpedo at a time captain!");
#endif
      return;
    }

  if (me->p_fuel < myship->s_plasmacost)
    {
#ifdef SHORT_PACKETS
      spwarning ("We don't have enough fuel to fire a plasma torpedo!", 22);
#else
      warning ("We don't have enough fuel to fire a plasma torpedo!");
#endif
      return;
    }
  if (me->p_flags & PFREPAIR)
    {
#ifdef SHORT_PACKETS
      spwarning ("We cannot fire while our vessel is undergoing repairs.", 23);
#else
      warning ("We cannot fire while our vessel is undergoing repairs.");
#endif
      return;
    }
  if ((me->p_cloakphase) && (me->p_ship.s_type != ATT))
    {
#ifdef SHORT_PACKETS
      spwarning ("We are unable to fire while in cloak, captain!", 24);
#else
      warning ("We are unable to fire while in cloak, captain!");
#endif
      return;
    }
  me->p_nplasmatorp++;
  me->p_fuel -= myship->s_plasmacost;
  me->p_wtemp += (myship->s_plasmacost / 10) - 8;	/* Heat weapons */
  for (i = me->p_no * MAXPLASMA, k = &plasmatorps[i];	/* Find a free plasma torp */
       i < me->p_no * MAXPLASMA + MAXPLASMA; i++, k++)
    {
      if (k->pt_status == PTFREE)
	break;
    }

  /* Setup data in new torp */
  k->pt_no = i;
  k->pt_status = type;
  k->pt_owner = me->p_no;
  k->pt_team = me->p_team;
  k->pt_x = me->p_x;
  k->pt_y = me->p_y;
  if ((myship->s_type == STARBASE) || (myship->s_type == ATT))
    k->pt_dir = course;
  else
    k->pt_dir = me->p_dir;
  k->pt_damage = myship->s_plasmadamage;
  k->pt_speed = myship->s_plasmaspeed;
  k->pt_war = me->p_hostile | me->p_swar;
  k->pt_fuse = myship->s_plasmafuse;
  k->pt_turns = myship->s_plasmaturns;
}
