/*
 * smessage.c

 */
#include "copyright.h"

#include <stdio.h>
#include <math.h>
#include <signal.h>
#include <ctype.h>
#include <string.h>
#include "defs.h"
#include "struct.h"
#include "data.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <limits.h>
#include <time.h>
#include <sys/time.h>

#include "planets.h"


void
copy80 (dest, addr_format, address, str)

     char *dest;
     char *addr_format;
     char *address;
     char *str;
{
  char addr_buf[16];
  int l;

  sprintf (addr_buf, addr_format, address);
  l = strlen (addr_buf);
  strncpy (dest, addr_buf, l);
  strncpy (&dest[l], str, 79 - l);
  dest[79] = '\0';
}

pmessage (str, recip, group, address)
     char *str;
     int recip;
     int group;
     char *address;
{
  struct message *cur;
  int mesgnum;

  if ((mesgnum = ++(mctl->mc_current)) >= MAXMESSAGE)
    {
      mctl->mc_current = 0;
      mesgnum = 0;
    }
  cur = &messages[mesgnum];
  cur->m_no = mesgnum;
  cur->m_flags = group;
  cur->m_recpt = recip;
  cur->m_from = 255;		/* From god */
  if (address && *address)
    copy80 (cur->m_data, "%-9s ", address, str);
  else
    {
      strncpy (cur->m_data, str, 79);
      cur->m_data[79] = 0;
    }
  cur->m_flags |= MVALID;
#ifdef SHORT_PACKETS
  cur->args[0] = DINVALID;	/* This is to prevent false  sending with SP_S_WARNING */
#endif
}

#define NUM_COMMANDS 28
static short captonlycomm[NUM_COMMANDS] =
{
  0, -1, -1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, -1, 1, 1, 0, 1, 1, 1};

static char *commands[NUM_COMMANDS] =
{
  "TEAM",			/* 0 */
  "HOME",			/* 1 */
  "AWAY",			/* 2 */
  "TIME",			/* 3 */
  "HELP",			/* 4 */

  "CAPTAIN",			/* 5 */
  "CONTINUE",			/* 6 */
  "FED",			/* 7 */
  "ROM",			/* 8 */
  "KLI",			/* 9 */
  "ORI",			/* 10 */
  "RESTART",			/* 11 */
  "START",			/* 12 */

  "PAUSE",			/* 13 */
  "NEWGALAXY",			/* 14 */
  "UNCAPTAIN",			/* 15 */
  "PAUSENOW",			/* 16 */
  "PASS",			/* 17 */
  "GAMETIME",			/* 18 */
  "FREE",			/* 19 */
  "QUEUE",			/* 20 */
  "RESTRICT",			/* 21 */
  "CONFINE",			/* 22 */
  "TIMEOUT",			/* 23 */
  "RESERVE",			/* 24 */
  "TNAME",			/* 25 */
  "REGISTER",			/* 26 */
  "ARMY",			/* 27 */
};


pmessage2 (str, recip, group, address, from)
     char *str;
     int recip;
     int group;
     char *address;
     unsigned char from;
{
  struct message *cur;
  int mesgnum;

  if (InlMsgParse (str, recip, group, address, from) == 1)
    return;

  if ((mesgnum = ++(mctl->mc_current)) >= MAXMESSAGE)
    {
      mctl->mc_current = 0;
      mesgnum = 0;
    }
  cur = &messages[mesgnum];
  cur->m_no = mesgnum;
  cur->m_flags = group;
  cur->m_recpt = recip;
  cur->m_from = from;
  if (address && *address)
    copy80 (cur->m_data, "%-9s ", address, str);
  else
    {
      strncpy (cur->m_data, str, 79);
      cur->m_data[79] = 0;
    }
  cur->m_flags |= MVALID;
#ifdef SHORT_PACKETS
  cur->args[0] = DINVALID;	/* This is to prevent false  sending with SP_S_WARNING */
#endif
}

int
doreserve (p, from)
     char *p;
     unsigned char from;
{
  int i, j;
  int resf;
  int noadd;
  int y, mm, d, h, m, dur;
  char temp[80];
  struct tm *tm_start;
  struct timeval tv_date;
  struct resentry res[MAXRES];

  /* let's do it this way: 
     reserve - prints help
     reserve all - prints all reservations
     reserve year month day hour min duration team - reserves a time
     TODO: cancel a reservation */
  p = strtok (NULL, "\0\n");
  if (p == NULL)
    {
      pmessage ("RESERVE all - lists all reservations", from, MINDIV, "");
      pmessage ("RESERVE all x - lists reservation #x in detail", from, MINDIV, "");
      pmessage ("RESERVE yy mm dd hh mm duration team_name - make a reservation (local time)", from, MINDIV, "");
      pmessage ("------- duration in minutes, hours assume 24 hour clock", from, MINDIV, "");
      return 1;
    }
  i = 0;
  noadd = 0;
  /* first we read in all the ones we have so far */
  resf = open (RESERVATIONS, O_RDONLY | O_CREAT);
  if (resf <= 0)
    {
      pmessage ("Error opening reservations file... sorry", from, MINDIV, "");
      return 1;
    }
  while (read (resf, (char *) &res[i], sizeof (struct resentry)) != NULL &&
	 i < MAXRES)
    {
      i++;
    }
  if (i == MAXRES)
    {
      pmessage ("Warning: Maximum number of reservations filled", from, MINDIV, "");
      noadd = 1;
    }
  close (resf);

  if (strstr (p, "all"))
    {
      if (i == 0)
	{
	  pmessage ("There are currently no reservations for this server", from, MINDIV, "");
	  return 1;
	}
      /* did we request a detailed outline ? */
      if (sscanf (p, "%s %d", temp, &j) == 2)
	{
	  if (j >= i || j < 0)
	    {
	      pmessage ("Request out of range", from, MINDIV, "");
	      return 1;
	    }
	  sprintf (temp, "On %s %d min for %s", ctime (&res[j].start_time),
		   (res[j].end_time - res[j].start_time) / 60,
		   res[j].team_name);
	  pmessage (temp, from, MINDIV, "");
	  sprintf (temp, "By %s@%s on %s", res[j].p_login,
		   res[j].p_monitor, ctime (&res[j].when_res));
	  pmessage (temp, from, MINDIV, "");
	  return 1;
	}
      for (j = 0; j < i; j++)
	{
	  sprintf (temp, "On %s %d min for %s (%s)", ctime (&res[j].start_time),
		   (res[j].end_time - res[j].start_time) / 60,
		   res[j].team_name, res[j].p_login);
	  pmessage (temp, from, MINDIV, "");
	}
      return 1;
    }

  if (sscanf (p, "%d %d %d %d %d %d %50s",
	      &y, &mm, &d, &h, &m, &dur, temp) != 7)
    {
      pmessage ("Improper usage ... Wrong number of arguments....", from, MINDIV, "");
      return 1;
    }

  /* there's probably a better way to handle white space in team_name
     but oh well - jmn */
  strcpy (res[i].team_name, strstr (p, temp));

  gettimeofday (&tv_date, (struct timezone *) 0);
  tm_start = localtime ((time_t *) & tv_date.tv_sec);
  /* make sure tm_start fields are reasonable */
  res[i].when_res = time (NULL);
  tm_start->tm_sec = 0;
  tm_start->tm_min = m;
  tm_start->tm_hour = h;
  tm_start->tm_mday = d;
  tm_start->tm_mon = mm - 1;
  tm_start->tm_year = y;
  tm_start->tm_isdst = -1;
  res[i].start_time = mktime (tm_start);
  res[i].end_time = res[i].start_time + dur * 60;

/*      pmessage(ctime(&res[i].when_res),from,MINDIV,"NOW:");
   pmessage(ctime(&res[i].start_time),from,MINDIV,"STRT:");
   pmessage(ctime(&res[i].end_time),from,MINDIV,"END:");
 */

  /* check for  conflicts */
  for (j = 0; j < i; j++)
    {
      if ((res[j].start_time <= res[i].start_time &&
	   res[j].end_time >= res[i].start_time) ||
	  res[j].start_time <= res[i].end_time &&
	  res[j].end_time >= res[i].end_time)
	{
	  sprintf (temp, "Sorry that time conflicts with '%s'", res[j].team_name);
	  pmessage (temp, from, MINDIV, "");
	  sprintf (temp, "On %s for %d min made by %s", ctime (&res[j].start_time),
		   (res[j].end_time - res[j].start_time) / 60,
		   res[j].p_login);
	  pmessage (temp, from, MINDIV, "");
	  return 1;
	}
    }

  strcpy (res[i].p_name, me->p_name);
  strcpy (res[i].p_login, me->p_login);
  strcpy (res[i].p_monitor, me->p_monitor);

  resf = open (RESERVATIONS, O_APPEND | O_WRONLY);
  if (resf < 0)
    {
      pmessage ("OOOOPS..... Error opening file for write.. aborting", from, MINDIV, "");
      close (resf);
      return 1;
    }
  if (write (resf, (char *) &res[i], sizeof (struct resentry)) < 0)
    {
      pmessage ("OOOOPS..... Error writing file.. aborting", from, MINDIV, "");
      close (resf);
      return 1;
    }
  close (resf);
  sprintf (temp, "Reserved for %s by %s", res[i].team_name, res[i].p_login);
  pmessage (temp, from, MINDIV, "");
  sprintf (temp, "On %s for %d min", ctime (&res[i].start_time), dur);
  pmessage (temp, from, MINDIV, "");

  return 1;
}


/* parse and deal with messages
   ** return 1 if the message was processed
   ** return 0 otherwise
 */
int 
InlMsgParse (str, recip, group, address, from)
     char *str;
     int recip;
     int group;
     char *address;
     unsigned char from;
{
  int FirstPickTeam, SecondPickTeam;
  char *TeamPtrs[2];
  int i;
  char buf[128], buf2[128];
  struct player *j;
  int tmp1, tmp2;
  char *p;
  char tleft, treq;
  char mename[3];		/* who am I ie. like F0 R1, etc. */

  TeamPtrs[0] = "Home";
  TeamPtrs[1] = "Away";
  sprintf (mename, "%c%c", teamlet[players[from].p_team],
	   shipnos[players[from].p_no]);

  if ((group != MINDIV) || (recip != from))
    return (0);

  /* kludge alert: we don't want special parseQuery commands in socket.c
     to generate a usage message here (added '#' -tsh) */
  if (strlen (str) == 1 && (*str == '?' || *str == '!' || *str == '#'))
    return 0;

  strcpy (buf, str);
  p = strtok (buf, " \t\n");
  if (p == NULL)
    {
      pmessage ("What was that?", from, MINDIV, fromserver (from));
      return (0);
    }

  for (i = NUM_COMMANDS - 1; i >= 0; i--)
    {
#ifdef CASE_SENSITIVE
      if (strcmp (p, commands[i]) == 0)
#else
      if (StrcmpI (p, commands[i]) == 0)
#endif
	break;
    }

#ifdef DTEST
  if (StrcmpI (p, "gotest") == 0)
    {
      captains[onHomeTeam ? 0 : 1] = from;
      for (i = 0; i < 2; i++)
	{
	  INLteamschosen[i] = 1 + i;
	  sprintf (teamname[i], "Team%d", i);
	  gamelenreq[i] = 4;
	  otlenreq[i] = 5;
	  startreq[i] = 1;
	}
      return (0);
    }
#endif


  if (i < 0)
    {
      char buf2[120];
      sprintf (buf2, "I don't understand %s, HELP for help", p);
      pmessage (buf2, from, MINDIV, fromserver (from));
      return (0);
    }

#ifdef DEBUG
  {
    char buf2[100];
    sprintf (buf2, "Command: %s(%d)", commands[i], i);
    warning (buf2);
  }
#endif

  if ((captonlycomm[i] == 1) && (captains[onHomeTeam ? 0 : 1] != from))
    {
      pmessage ("Only the captain can do that!!", from, MINDIV,
		fromserver (from));
      return (1);
    }

  if (captonlycomm[i] == -1)
    {
      pmessage ("Sorry - that command is disabled.", from, MINDIV,
		fromserver (from));
      return (1);
    }

  switch (i)
    {
    case 0:			/* TEAM */
      sprintf (buf, "You are on the %s team", TeamPtrs[1 - onHomeTeam]);
      pmessage (buf, from, MINDIV, fromserver (from));
      break;
    case 1:			/* HOME */
    case 2:			/* AWAY */
      if (status->tourn)
	{
	  pmessage ("No switching during after game start!", from, MINDIV,
		    fromserver (from));
	  return 1;
	}

      if ((onHomeTeam && i == 1) || (!onHomeTeam && i == 2))
	{
	  pmessage ("You're already on that team", from, MINDIV,
		    fromserver (from));
	  return 1;
	}

      if (realNumShips ((i == 1) ? 1 : 0) >= INL_TEAMAX)
	{
	  pmessage ("Sorry, that team is full.", from, MINDIV,
		    fromserver (from));
	  return 1;
	}

      onHomeTeam = (i == 1) ? 1 : 0;
      if (captains[1 - onHomeTeam] == from)
	free_captain ();
      me->p_team = INLteams[1 - onHomeTeam];

      if (onHomeTeam)
	{
	  pmessage ("You are now on the home team", from, MINDIV,
		    fromserver (from));
	}
      else
	{
	  pmessage ("You are now on the away team", from, MINDIV,
		    fromserver (from));
	}
      return 1;
      break;
    case 3:			/* TIME */
      switch (*INL_period)
	{
	case 0:
	  p = "Regulation play";
	  break;
	case 1:
	  p = "Overtime";
	  break;
	}
      sprintf (buf, "Time left: %3.3f minutes in %s", ((float) *INL_timeleft) / 600.0, p);
      pmessage (buf, from, MINDIV, fromserver (from));
      break;
    case 4:			/* HELP */
      for (i = 1; i < NUM_COMMANDS; i++)
	{
	  if (!captonlycomm[i])
	    {
	      pmessage (commands[i], me->p_no, MINDIV, "Help: ");
	    }
	}
      if (captains[onHomeTeam ? 0 : 1] != me->p_no)
	return 1;
      pmessage ("Captain only commands:", me->p_no, MINDIV, "");
      for (i = 1; i < NUM_COMMANDS; i++)
	{
	  if (captonlycomm[i])
	    pmessage (commands[i], me->p_no, MINDIV, "Captain Help: ");
	}
      return 1;
      break;
    case 5:			/* CAPTAIN */
      if (captains[onHomeTeam ? 0 : 1] == -1)
	{
	  captains[onHomeTeam ? 0 : 1] = from;
	  sprintf (buf, "%s(%c) is captain of the %s team",
		   players[from].p_name,
		   players[from].p_mapchars[1],
		   TeamPtrs[1 - onHomeTeam]);
	  pmessage (buf, 0, MALL, "");
	}
      else
	{
	  if (players[captains[onHomeTeam ? 0 : 1]].p_status != PALIVE)
	    {
	      sprintf (buf, "%s has died and been overthrown as captain!",
		       players[captains[onHomeTeam ? 0 : 1]].p_mapchars);
	      pmessage (buf, players[from].p_team, MTEAM, "");
	      captains[onHomeTeam ? 0 : 1] = -1;
	    }
	  else
	    {
	      sprintf (buf, "%s is already captain for your team",
		       players[captains[onHomeTeam ? 0 : 1]].p_mapchars);
	      pmessage (buf, from, MINDIV, fromserver (from));
	    }
	}
      break;
    case 6:			/* CONTINUE */
      continuereq[onHomeTeam ? 0 : 1] = 1;
      sprintf (buf, "%s team (%s) requests that the game continue",
	   onHomeTeam ? "Home" : "Away", mename);
      pmessage (buf, 0, MALL, "");
      break;
    case 7:			/* FED */
    case 8:			/* ROM */
    case 9:			/* KLI */
    case 10:			/* ORI */
      i = 1 << (i - 7);		/* team chosen */

      /* set PickTeam to teamarray index for respective teams */
      if (status->firstpickpassed)
	{
	  FirstPickTeam = 0;	/* home team */
	  SecondPickTeam = 1;	/* visitors */
	}
      else
	{
	  FirstPickTeam = 1;	/* visitors pick first */
	  SecondPickTeam = 0;	/* home picks first */
	}

      /* note that onHomeTeam is reversed from INLteamsarray index */
      if (onHomeTeam == SecondPickTeam)
	{
	  /* the team that picks first */

	  /* allow changing of team up intil second team chooses */
	  if (INLteamschosen[SecondPickTeam] != 0)
	    {
	      sprintf (buf, "Team is already set to %s",
		       teamshort[INLteamschosen[FirstPickTeam]]);
	      pmessage (buf, from, MINDIV, fromserver (from));
	      return 1;
	    }
	  INLteamschosen[FirstPickTeam] = i;
	  sprintf (buf, "%s team (%s) chooses %s", TeamPtrs[FirstPickTeam],
		   mename, teamshort[i]);
	  pmessage (buf, 0, MALL, "");
	  return 1;
	}
      else
	{
	  /* the team that picks second */

	  if (INLteamschosen[FirstPickTeam] == 0)
	    {
	      pmessage ("Wait for the other team to choose first!",
			from, MINDIV, "");
	      return 1;
	    }
	  if (INLteamschosen[FirstPickTeam] == i)
	    {
	      sprintf (buf, "The %s has already been chosen. Pick another.",
		       teamshort[INLteamschosen[FirstPickTeam]]);
	      pmessage (buf, from, MINDIV, fromserver (from));
	      return 1;
	    }

	  if (i == diagnolTeams[INLteamschosen[FirstPickTeam]])
	    {
	      sprintf (buf, "Please choose one of the races adjacent to %s",
		       teamshort[INLteamschosen[FirstPickTeam]]);
	      pmessage (buf, from, MINDIV, fromserver (from));
	      return 1;
	    }
	  INLteamschosen[SecondPickTeam] = i;
	  sprintf (buf, "%s team (%s) chooses %s", TeamPtrs[SecondPickTeam],
		   mename, teamshort[i]);
	  pmessage (buf, 0, MALL, "");
	  return 1;
	}
      break;
    case 11:			/* RESTART */
      if (!(pausereq[0] && pausereq[1]))	/* Tom`s Request */
	{
	  restartreq[onHomeTeam ? 0 : 1] = 1;
	  sprintf (buf, "%s team (%s) requests that the game be restarted",
		   onHomeTeam ? "Home" : "Away", mename);
	  pmessage (buf, 0, MALL, "");
	}
      else
	{
	  pmessage ("Cannot RESTART until game is CONTINUE(d)", from, MINDIV, fromserver (from));
	}
      break;
    case 12:			/* START */
      if ((INLteamschosen[0] != 0) && (INLteamschosen[1] != 0))
	{
	  if (strlen (teamname[onHomeTeam ? 0 : 1]) < 1)
	    {
	      pmessage ("You must select a team name first", from, MINDIV,
			fromserver (from));
	      return 1;
	    }
	  startreq[onHomeTeam ? 0 : 1] = 1;
	  sprintf (buf, "%s (%s) (%s) is ready to start", teamname[onHomeTeam ? 0 : 1],
		   onHomeTeam ? "Home" : "Away", mename);
	  pmessage (buf, 0, MALL, "");
	}
      else
	{
	  pmessage ("Teams are still being chosen", from, MINDIV,
		    fromserver (from));
	}
      break;
    case 13:			/* PAUSE */
      if (pausereq[onHomeTeam ? 0 : 1] == 1) 
        {
          pmessage ("**********************************************************", 0, MALL, "");
	  sprintf (buf, "%s team (%s) cancels their PAUSE request.",
		onHomeTeam ? "Home" : "Away", mename);
      	  pmessage (buf, 0, MALL, "");
          pmessage ("**********************************************************", 0, MALL, "");
	  pausereq[onHomeTeam ? 0: 1] = 0;
	  break;
	}
      else {
      pausereq[onHomeTeam ? 0 : 1] = 1;
      pmessage ("**********************************************************", 0, MALL, "");
      sprintf (buf, "%s team (%s) requests that the game be paused",
	       onHomeTeam ? "Home" : "Away", mename);
      pmessage (buf, 0, MALL, "");
      pmessage ("**********************************************************", 0, MALL, "");
      pmessage ("**********************************************************", 0, MALL, "");
      }
      break;
    case 14:			/* NEWGALAXY */
      newgalaxyreq[onHomeTeam ? 0 : 1] = 1;
      sprintf (buf,
	       "%s team (%s) requests that the galaxy be rebuilt and the game restarted",
	       onHomeTeam ? "Home" : "Away", mename);
      pmessage (buf, 0, MALL, "");
      break;
    case 15:			/* UNCAPTAIN */
      /* I can't take the responsibility!!!!!!!!! */
      if (captains[onHomeTeam ? 0 : 1] == from)
	{
	  free_captain ();
	}
      else
	{
	  pmessage ("You aren't the captain!", from, MINDIV, fromserver (from));
	}
      break;
    case 16:			/* PAUSENOW */
      pausereq[0] = 1;
      pausereq[1] = 1;
      sprintf (buf, "%s team (%s) has paused the game with a PAUSENOW",
	       onHomeTeam ? "Home" : "Away", mename);
      pmessage (buf, 0, MALL, "");
      break;
    case 17:			/* PASS */
      /* pass first team choice off to visiting team */

      if (onHomeTeam)
	{
	  pmessage (" Hey!! Only the Away team captain uses this!",
		    from, MINDIV, fromserver (from));
	  return 1;
	}

      /* allow passing up until  home team picks a race */
      if (INLteamschosen[0] != 0 /*|| INLTeamschosen[1] */ )
	{
	  pmessage ("Teams have already been chosen!",
		    from, MINDIV, fromserver (from));
	  return 1;
	}
      if (status->firstpickpassed == 1)
	{
	  sprintf (buf,
		   "Away team (%s) decides to keep first race choice after all!", mename);
	  pmessage (buf, 0, MALL, "");
	  status->firstpickpassed = 0;
	}
      else
	{
	  sprintf (buf,
	   "Away team (%s) passes first race choice to home team!", mename);
	  pmessage (buf, 0, MALL, "");
	  status->firstpickpassed = 1;
	  INLteamschosen[1] = 0;	/* make sure visiting team is cleared! */
	}
      break;
    case 18:			/* GAMETIME */

      if (status->tourn)
	{
	  pmessage ("You can't change time in tournament!", from, MINDIV,
		    fromserver (from));
	  return 1;
	}
      p = strtok (NULL, " \t");
      if (p == NULL)
	{
	  sprintf (buf, "Current game set to %d min with %d min overtime",
		   INL_game_length / 600, INL_overtime_length / 600);
	  pmessage (buf, 0, MALL, "");
	  return 1;
	}
      if (p != NULL && captains[onHomeTeam ? 0 : 1] != from)
	{
	  pmessage ("Only the captain can change the gametime", from, MINDIV, fromserver (from));
	  return 1;
	}
      tmp1 = abs (atoi (p));
      p = strtok (NULL, " \t");
      if (p != NULL)
	{
	  tmp2 = abs (atoi (p));
	}
      else
	{
	  tmp2 = INL_overtime_length / 600;
	}
      sprintf (buf, "%s team (%s) requests a game of %d min with %d min overtime.",
	       onHomeTeam ? "Home" : "Away", mename, tmp1, tmp2);
      pmessage (buf, 0, MALL, "");
      gamelenreq[onHomeTeam ? 0 : 1] = tmp1;
      otlenreq[onHomeTeam ? 0 : 1] = tmp2;
      break;
    case 19:			/* FREE */
      p = strtok (NULL, " \t");
      if (p == NULL)
	{
	  pmessage ("Which slot do you want to free?", from, MINDIV,
		    fromserver (from));
	  pmessage ("This command should be used with caution.. Can confuse server", from, MINDIV, fromserver (from));
	  return 1;
	}
      tmp1 = atoi (p);
      if ((tmp1 == 0) && (*p != '0'))
	{
	  tmp2 = *p;
	  if (tmp2 >= 'a' && tmp2 <= 'z')
	    tmp1 = tmp2 - 'a' + 10;
	  if (tmp1 >= MAXPLAYER || tmp1 < 0)
	    {
	      sprintf (buf2, "What slot? Is there a slot %s? Since when?", p);
	      pmessage (buf2, from, MINDIV, fromserver (from));
	      return 1;
	    }
	}
      if (players[tmp1].p_status == PALIVE || players[tmp1].p_status == POBSERV)
	{
	  sprintf (buf2, "Slot %d is still alive!!!", tmp1);
	  pmessage (buf2, from, MINDIV, fromserver (from));
	  return 1;
	}
      else
	{
	  sprintf (buf2, "%s team (%s) issued a free %s command!",
		   onHomeTeam ? "Home" : "Away", mename, p);
	  pmessage (buf2, 0, MALL, "");
#ifdef DS
	  if (!players[tmp1].p_process ||
	      kill (players[tmp1].p_process, SIGTERM) < 0)
	    /* NOTE: we only bzero if all else fails -- otherwise
	       we run the risk of zero'ing his stats if his process
	       recovers later */
#endif
	    bzero (&(players[tmp1]), sizeof (struct player));
	  players[tmp1].p_status = PFREE;
	  sprintf (buf2, "Freed slot %d.....", tmp1);
	  pmessage (buf2, 0, MALL, "");
	  return 1;
	}
      break;
    case 20:			/* QUEUE - query the queues */
      tmp1 = Queue_Length (1);	/* home queue I think */
      sprintf (buf2, "Home queue probably has %d people.", tmp1);
      pmessage (buf2, from, MINDIV, fromserver (from));
      tmp1 = Queue_Length (2);	/* away queue I think */
      sprintf (buf2, "Away queue probably has %d people.", tmp1);
      pmessage (buf2, from, MINDIV, fromserver (from));
      tmp1 = Queue_Length (0);
      sprintf (buf2, "Total queue length is %d people.", tmp1);
      pmessage (buf2, from, MINDIV, fromserver (from));
      return 1;
      break;
    case 21:			/* RESTRICT - restrict a command to captain only */
      if (captains[onHomeTeam ? 0 : 1] != from)
	{
	  pmessage ("Only the captain can restrict commands", from, MINDIV,
		    fromserver (from));
	  return 1;
	}

      p = strtok (NULL, " \t");
      if (p == NULL)
	{
	  pmessage ("Which command do you want me to restrict?", from, MINDIV, fromserver (from));
	  return 1;
	}
      for (i = NUM_COMMANDS - 1; i >= 0; i--)
	{
#ifdef CASE_SENSITIVE
	  if (strcmp (p, commands[i]) == 0)
#else
	  if (StrcmpI (p, commands[i]) == 0)
#endif
	    break;
	}
      if (i < 0)
	{
	  sprintf (buf2, "I don't know command %s so how can I restrict it?", p);
	  pmessage (buf2, from, MINDIV, fromserver (from));
	  return (1);
	}
      captonlycomm[i] = (captonlycomm[i] == 0) ? 1 : 0;
      sprintf (buf2, "%s command toggled to %s", p,
	       captonlycomm[i] ? "Captain only usage." : "general usage.");
      pmessage (buf2, players[from].p_team, MTEAM, "");
      return 1;
      break;
    case 22:			/* CONFINE */
      if (status->tourn)
	{
	  pmessage ("Weenie. You can't do a confine in tmode!!", from, MINDIV,
		    fromserver (from));
	  return 1;
	}
      confinereq[onHomeTeam ? 0 : 1] = (confinereq[onHomeTeam ? 0 : 1] == 0) ? 1 : 0;
      sprintf (buf2, "%s team (%s) has toggled confine %s (confine is an OR function)",
	       onHomeTeam ? "Home" : "Away", mename,
	       confinereq[onHomeTeam ? 0 : 1] ? "on" : "off");
      pmessage (buf2, 0, MALL, "");
      break;
    case 23:			/* TIMEOUT */
      if (!status->tourn)
	{
	  pmessage ("Timeout??  You aren't even PLAYING!", from, MINDIV,
		    fromserver (from));
	  return 1;
	}


      /* we set the high bit high to indicate that we did a request 
         the low 7 bits count how many timeouts we have left */
      tleft = timeoutreq[onHomeTeam ? 0 : 1] & 0x7f;
      treq = timeoutreq[onHomeTeam ? 0 : 1] & 0x80;

      if (treq)
	{
	  sprintf (buf2, "%s team (%s) has cancelled their timeout request!",
		   onHomeTeam ? "Home" : "Away", mename);
	  treq = 0;
	  tleft++;
	  pmessage (buf2, 0, MALL, "");
	}
      else if (!treq && (tleft > 0))
	{
	  pmessage ("**********************************************************", 0, MALL, "");
	  pmessage ("**********************************************************", 0, MALL, "");
	  sprintf (buf2, "%s team (%s) has called a timeout!",
		   onHomeTeam ? "Home" : "Away", mename);
	  pmessage (buf2, 0, MALL, "");
	  pmessage ("**********************************************************", 0, MALL, "");
	  pmessage ("**********************************************************", 0, MALL, "");
	  treq = 0x80;
	  tleft--;
	}
      else
	{
	  pmessage ("You have exhausted your timeout allowance for this game.", from,
		    MINDIV, fromserver (from));
	  return 1;
	}
      timeoutreq[onHomeTeam ? 0 : 1] = treq + tleft;
/*      sprintf(buf2,"treq=%d, tleft=%d, t=%d trnd=%d",treq,tleft,timeoutreq[onHomeTeam ? 0 : 1],
   (timeoutreq[0] | timeoutreq[1]) & 0x80);
   pmessage(buf2,from,MINDIV,fromserver(from));
 */
      break;
    case 24:			/* RESERVE */
      return doreserve (p, from);
      break;
    case 25:			/* TNAME */
      p = strtok (NULL, "\0\n");
      if (p == NULL)
	{
	  pmessage ("TNAME team_name - identify your team name", from, MINDIV,
		    fromserver (from));
	  return 1;
	}
      if (status->tourn)
	{
	  pmessage ("You can't change your team name in tmode (I bet you're losing)", from, MINDIV, fromserver (from));
	  return 1;
	}
      strncpy (teamname[onHomeTeam ? 0 : 1], p, 20);
      teamname[onHomeTeam ? 0 : 1][19] = '\0';
      sprintf (buf2, "Your team name set to '%s'.", teamname[onHomeTeam ? 0 : 1]);
      pmessage (buf2, from, MINDIV, fromserver (from));
      return 1;
      break;
    case 26:			/* REGISTER */
      regreq[onHomeTeam ? 0 : 1] ^= 0xff;
      sprintf (buf2, "%s team (%s) claims this is an %s game",
	       onHomeTeam ? "Home" : "Away", mename,
	       regreq[onHomeTeam ? 0 : 1] ? "Official" : "Unofficial");
      pmessage (buf2, 0, MALL, "");
      if (regreq[0] && regreq[1])
	pmessage ("Game is official.. will be automagically registered", 0, MALL, "");
      return 1;
      break;
    case 27:			/* ARMY */
      if (status->tourn)
	{
	  pmessage ("You can't change number of starting armies during a tournament!", from, MINDIV, fromserver (from));
	  return 1;
	}
      p = strtok (NULL, " \t");
      if (p == NULL)
	{
	  sprintf (buf, "Starting army set to %d.",
		   *initial_armies);
	  pmessage (buf, 0, MALL, "");
	  return 1;
	}
      if (p != NULL && captains[onHomeTeam ? 0 : 1] != from)
	{
	  pmessage ("Only the captain can request a change in starting armies!", from, MINDIV, fromserver (from));
	  return 1;
	}
      tmp1 = abs (atoi (p));
      sprintf (buf, "%s team (%s) requests starting armies be set to %d.",
	       onHomeTeam ? "Home" : "Away", mename, tmp1);
      pmessage (buf, 0, MALL, "");
      setarmyreq[onHomeTeam ? 0 : 1] = tmp1;
      return 1;
      break;

    }
  return 1;
}

free_captain ()			/* we also call this from exitGame */
{
  char buf[128];
  int team_index;

  team_index = onHomeTeam ? 0 : 1;
  sprintf (buf, "%s(%d) is no longer captain of the %s team",
	   me->p_name,
	   me->p_no,
	   onHomeTeam ? "home" : "away");
  pmessage (buf, 0, MALL, "");
  captains[team_index] = -1;
  pausereq[team_index] = 0;
  continuereq[team_index] = 0;
  startreq[team_index] = 0;
  restartreq[team_index] = 0;
  newgalaxyreq[team_index] = 0;
}


/* case insensitive strncmp */
int 
StrcmpI (s1, s2)
     char *s1, *s2;
{
  char c1;
  char c2;
  char *p1;
  char *p2;
  int count = 0;
  p1 = s1;
  p2 = s2;

  if (p1 == NULL || p2 == NULL)
    return -1;

  while (1)
    {
      c1 = islower (*p1) ? toupper (*p1) : *p1;
      c2 = islower (*p2) ? toupper (*p2) : *p2;
      if (c1 != c2 || c1 == 0 || c2 == 0)
	return c1 - c2;
      p1++;
      p2++;
      count++;
    }
}

char *
fromserver (from)

     int from;
{
  /* extend as needed */
  static char addr[10];
  sprintf (addr, "SRV->%c%c", players[from].p_mapchars[0],
	   players[from].p_mapchars[1]);
  return addr;
}
