/*
 * torp.c
 */
#include "copyright.h"

#include <stdio.h>
#include <sys/types.h>
#ifdef NOTIMEB
#include <sys/time.h>
#else
#include <sys/timeb.h>
#endif
#include <sys/ipc.h>
#include "defs.h"
#include "struct.h"
#include "data.h"

/* Launch torp */

/*
** If a set of given conditions are met, fire a single torp in direction
** course.  Type is used because robots are allowed to shoot straight.
** Torps sent with status TMOVE wobble a bit.  TSTRAIGHT torps (fired
** by robots) move in a straight line.
**
** torp->t_fuse is the life of the torpedo.  It is set here based on
** a random function.  Torps currently live two to five seconds.
*/

#ifdef NOTIMEB
static struct timeval lasttime;
static struct timeval curtime;
#else
static struct timeb lasttime;
static struct timeb curtime;
#endif
#define GAP 100
long rrandom();

ntorp(course, type)
unsigned char course;
int type;
{
    register int i;
    register struct torp *k;

    if (me->p_flags & PFWEP) {
#ifdef SHORT_PACKETS
	spwarning("Torpedo launch tubes have exceeded maximum safe temperature!", 25);
#else
	warning("Torpedo launch tubes have exceeded maximum safe temperature!");
#endif
	return;
    }

#ifdef NOTIMEB            /* 11/6/93 SK replacing ftime with gettimofday */
#define TMGAP (GAP*1000)
    gettimeofday(&curtime,0);
    if ((curtime.tv_sec == lasttime.tv_sec && curtime.tv_usec <
            lasttime.tv_usec+TMGAP) ||
        (curtime.tv_sec == lasttime.tv_sec+1 && curtime.tv_usec+1000000-
            lasttime.tv_usec < TMGAP)) {
            return;
        } else {
        lasttime.tv_sec = curtime.tv_sec;
        lasttime.tv_usec = curtime.tv_usec;
    }
#else
    ftime(&curtime);
    if ((curtime.time == lasttime.time && curtime.millitm < lasttime.millitm+GAP) ||
	(curtime.time == lasttime.time+1 && curtime.millitm+1000-lasttime.millitm < GAP)) {
	return;
    } else {
	lasttime.time = curtime.time;
	lasttime.millitm = curtime.millitm;
    }
#endif

    if (me->p_ntorp == MAXTORP) {
#ifdef SHORT_PACKETS
	spwarning("Our computers limit us to having 8 live torpedos at a time captain!", 26);
#else
	warning("Our computers limit us to having 8 live torpedos at a time captain!");
#endif
	return;
    }
    if (me->p_fuel < myship->s_torpcost) {
#ifdef SHORT_PACKETS
	spwarning("We don't have enough fuel to fire photon torpedos!", 27);
#else
	warning("We don't have enough fuel to fire photon torpedos!");
#endif
	return;
    }
    if (me->p_flags & PFREPAIR) {
#ifdef SHORT_PACKETS
	spwarning("We cannot fire while our vessel is in repair mode.", 28);
#else
	warning("We cannot fire while our vessel is in repair mode.");
#endif
	return;
    }
    if ((me->p_cloakphase) && (me->p_ship.s_type != ATT)) {
#ifdef SHORT_PACKETS
	spwarning("We are unable to fire while in cloak, captain!", 29);
#else
	warning("We are unable to fire while in cloak, captain!");
#endif
	return;
    }

/* change TC 12/9/90 -- my attempt at torp angle stuff */


    if (0 && ((me->p_ship).s_type != STARBASE)) {
	int delta;
	if ((delta = ((int) me->p_dir - (int) course)) < 0)
	    delta = -delta;
	if (delta > topgun) { /* note: 128 = 180 degrees left/right */
#ifdef SHORT_PACKETS
	    spwarning("We only have forward mounted cannons.", 30);
#else
	    warning("We only have forward mounted cannons.");
#endif
	    return;
	}
    } /* end if topgun */

    me->p_ntorp++;
    me->p_fuel -= myship->s_torpcost;
    me->p_wtemp += (myship->s_torpcost / 10) - 10;	/* Heat weapons */
    for (i = me->p_no * MAXTORP, k = &torps[i];		/* Find a free torp */
	i < me->p_no * MAXTORP + MAXTORP; i++, k++) {
	    if (k->t_status == TFREE)
		break;
    }

    /* Setup data in new torp */
    if (type>TSTRAIGHT || type<TFREE) type=TMOVE;
    k->t_no = i;
    k->t_status = type;
    k->t_owner = me->p_no;
    k->t_team = me->p_team;
    k->t_x = me->p_x;
    k->t_y = me->p_y;
    k->t_dir = course;
    k->t_damage = myship->s_torpdamage;
    k->t_speed = myship->s_torpspeed;
    k->t_war = me->p_hostile | me->p_swar;
    k->t_fuse = myship->s_torpfuse + (rrandom() % 20);
    k->t_turns = myship->s_torpturns;
    k->t_whodet = NODET;
}
