/* snidstats.c
 */

#include <stdio.h>
#include <string.h>
#include <sys/file.h>
#include <sys/types.h>
#include "defs.h"
#include "struct.h"
#include "data.h"
#include "newstats.h"
#include "../version.h"
#include "../patchlevel.h"

static FILE *outf;
static FILE *sfd;

char *BAD_HEADER = "Header info is wrong (try -o?)";
char *TRUNCATED = "The file has been truncated";

static char *tteamshort[9] =
{"IND", "FED", "ROM", "X", "KLI", "X", "X", "X", "ORI"};
static int rremap[9] =
{0, 1, 2, -1, 3, -1, -1, -1, 4};
static int doteam[NUMTEAM + 1] =
{0, 0, 0, 0, 0};
static int lflag;
static char *Prog_Name = "(???)";
static int bflag = 0;

main (argc, argv)
     int argc;
     char **argv;
{
  register int i, i2;
  int x = 0, k = 0;
  char tmp;
  char *tmpc;
  FILE *cnqf;
  FILE *fd;
  struct st_new_stats *nstatb;
  struct st_new_stats *totalb;
  int teams_set;
  char *p;
  int old_flag = 0;		/* !! */
  char ibuf[513];
  int bufsize;
  long team_1, team_2;
  int struct_size, vers_curr, vers_low, byteswapcheck;
#ifdef SOLARIS2
  extern char *strchr ();
#else
  extern char *index ();
#endif

  Prog_Name = argv[0];
  teams_set = 0;
  if (argc > 1 && *argv[1] == '-')
    {
      for (p = argv[1] + 1; *p; p++)
	{
	  switch (*p)
	    {
	    case 'o':
	      old_flag = 1;;
	      break;
	    case 'F':
	      doteam[rremap[FED]] = 1;
	      teams_set = 1;
	      break;
	    case 'R':
	      doteam[rremap[ROM]] = 1;
	      teams_set = 1;
	      break;
	    case 'K':
	      doteam[rremap[KLI]] = 1;
	      teams_set = 1;
	      break;
	    case 'O':
	      doteam[rremap[ORI]] = 1;
	      teams_set = 1;
	      break;
	    case 'A':
	      doteam[1] = doteam[2] = doteam[3] = doteam[4] = 1;
	      break;
	    case 'b':
	      bflag = 1;
	      break;
	    case 'l':
	      lflag = 1;
	      break;
	    default:
	      usage ();
	    }
	}
      argc--, argv++;
    }

#ifdef notdef
  if (!teams_set)
    doteam[1] = doteam[2] = doteam[3] = doteam[4] = 1;
#endif

  if ((argc != 2) && (argc != 3))
    {
      usage ();
    }

  sfd = fopen (argv[1], "r");
  if (argc == 3)
    {
      outf = fopen (argv[2], "w");
    }
  else
    outf = stderr;

  if (outf == NULL)
    {
      fprintf (stderr, "Error opening output file");
      exit (1);
    }

  fprintf (outf, "Snidly style stats - v%d.\n", NSTATS_VERSION);

  if (!old_flag)
    {
      /* read in header info */
      if (!fgets (ibuf, sizeof (ibuf), sfd))
	formaterror (TRUNCATED);

#ifdef SOLARIS2
      if ((p = strchr (ibuf, ':')))
#else
      if ((p = index (ibuf, ':')))
#endif
	{
	  *p++ = 0;
	  if (strcmp ("Netrek INL timestats, server", ibuf))
	    formaterror (BAD_HEADER);
	  fprintf (outf, "\tServer:%s", p);
	}
      else
	formaterror (BAD_HEADER);
      if (!fgets (ibuf, sizeof (ibuf), sfd))
	formaterror (TRUNCATED);
      fprintf (outf, "\t%s", ibuf);
      if (!fgets (ibuf, sizeof (ibuf), sfd))
	formaterror (TRUNCATED);
      fprintf (outf, "\t%s", ibuf);
      if (!fgets (ibuf, sizeof (ibuf), sfd))
	formaterror (TRUNCATED);
      if (strcmp ("The rest of this file is binary data.\n", ibuf))
	formaterror (BAD_HEADER);

      fread (&byteswapcheck, sizeof (long), 1, sfd);
      byteswapcheck = ntohl (byteswapcheck);
      if (byteswapcheck != NSTATS_BYTESWAP_MAGIC)
	formaterror ("byte swap");
      fread (&vers_low, sizeof (long), 1, sfd);
      vers_low = ntohl (vers_low);
      fread (&vers_curr, sizeof (long), 1, sfd);
      vers_curr = ntohl (vers_curr);
      fread (&struct_size, sizeof (long), 1, sfd);
      struct_size = ntohl (struct_size);
      fread (&team_1, sizeof (long), 1, sfd);
      team_1 = ntohl (team_1);
      if (fread (&team_2, sizeof (long), 1, sfd) <= 0)
	  formaterror (TRUNCATED);
      team_2 = ntohl (team_2);

      if (!teams_set)
	{
	  if (team_1 > NUMTEAM || team_2 > NUMTEAM)
	    formaterror ("team numbers are bad");
	  doteam[team_1] = 1;
	  doteam[team_2] = 1;
	  teams_set = 1;
	}

      if (vers_curr > NSTATS_VERSION && vers_low > NSTATS_COMPAT)
	{
	  fprintf (stderr,
		   "this program (v%d) is too old for that file (v%d)",
		   NSTATS_VERSION, vers_curr);
	  exit (-1);
	}
      if (vers_curr < NSTATS_VERSION && vers_low < NSTATS_COMPAT)
	{
	  fprintf (stderr,
		   "this program (v%d) is too new for that file (v%d)",
		   NSTATS_VERSION, vers_curr);
	  exit (-1);
	}
      if (vers_curr != NSTATS_VERSION)
	fprintf (stderr,
	      "note: data file version (v%d) is not program version (%d)\n",
		 vers_curr, NSTATS_VERSION);
    }
  else
    {
      struct_size = sizeof (struct st_new_stats);
      /* XXX preserver old stat struct ?? */
    }

  if (struct_size < sizeof (struct st_new_stats))
      bufsize = sizeof (struct st_new_stats);
  else
    bufsize = struct_size;

  nstatb = (struct st_new_stats *) malloc (bufsize);
  totalb = (struct st_new_stats *) malloc (bufsize);

  if (!nstatb || !totalb)
    {
      fprintf (stderr, "Can't create stat struct buffer of %d bytes\n",
	       bufsize);
      exit (1);
    }

  bzero (nstatb, bufsize);
  bzero (totalb, bufsize);

  i = 0;
  while (fread (nstatb, struct_size, 1, sfd) > 0)
    {
      if (!old_flag || bflag)
	swapb (nstatb, struct_size / sizeof (long));
      if (nstatb->time == 0 && nstatb->period == 1)
	{
	  if (i)
	    {
	      i = 0;
	      fprintf (outf, " --- GAME RESTARTED ---\n");
	    }
	  bzero (totalb, bufsize);
	}
      else if (nstatb->time == 0)
	i = 4;
      if (!i && !teams_set)
	{
	  if (nstatb->team[1].bomb.num)
	    doteam[1] = 1;
	  if (nstatb->team[2].bomb.num)
	    doteam[2] = 1;
	  if (nstatb->team[3].bomb.num)
	    doteam[3] = 1;
	  if (nstatb->team[4].bomb.num)
	    doteam[4] = 1;
	}
      if (!(i % 4))
	{
	  header ("time");
	}
      i++;
      process (nstatb, outf, totalb);
    }

  fprintf (outf, "\n");
  header ("TOTAL");

  print_statline (outf, totalb, 1);
  print_statline (outf, totalb, 2);
  print_statline (outf, totalb, 3);
  print_statline (outf, totalb, 4);

  print_expl (outf);

  fclose (sfd);
  fclose (outf);
}

process (nstats, fd, total)
     struct st_new_stats *nstats;
     FILE *fd;
     struct st_new_stats *total;
{
  int i;

  if (nstats->period > 1)
    fprintf (fd, "%3d-OT%d\n", nstats->time, nstats->period - 2);
  else if (nstats->period > 0)
    fprintf (fd, "%3d-OT\n", nstats->time);
  else
    fprintf (fd, "%3d\n", nstats->time);
  /* Don't bother with indi planets */
  for (i = 1; i <= NUMTEAM; i++)
    {
      print_statline (fd, nstats, i);

      total->team[i].pop.check += nstats->team[i].pop.check;
      total->team[i].pop.hit += nstats->team[i].pop.hit;
      total->team[i].pop.num += nstats->team[i].pop.num;
      total->team[i].plague.hit += nstats->team[i].plague.hit;
      total->team[i].plague.count += nstats->team[i].plague.count;
      if (nstats->team[i].plague.low > 0 &&
	  (total->team[i].plague.low <= 0 ||
	   total->team[i].plague.low > nstats->team[i].plague.low))
	{
	  total->team[i].plague.low = nstats->team[i].plague.low;
	}
      total->team[i].bomb.num += nstats->team[i].bomb.num;
      total->team[i].drop.num += nstats->team[i].drop.num;
      total->team[i].carried += nstats->team[i].carried;
      total->team[i].player.num += nstats->team[i].player.num;
      total->team[i].player.lowfuel += nstats->team[i].player.lowfuel;
      total->team[i].player.damaged += nstats->team[i].player.damaged;
    }
}

print_statline (fd, nstats, tno)
     FILE *fd;
     struct st_new_stats *nstats;
     int tno;
{
  if (lflag)
    long_statline (fd, nstats, tno);
  else
    short_statline (fd, nstats, tno);

}

short_statline (fd, nstats, tno)
     FILE *fd;
     struct st_new_stats *nstats;
     int tno;
{
  int chk;

  if (!doteam[tno])
    return;

  chk = nstats->team[tno].pop.check;
  if (chk < 1)
    chk = 1;

  fprintf (fd, "   %3s  %4d %3d %3d %04d  %3d %3d %3d  %3d %3d %3d  %4d\n",
	   tteamshort[1 << tno - 1],
	   nstats->team[tno].pop.check,
	   nstats->team[tno].pop.hit,
	   nstats->team[tno].pop.num,
	   nstats->team[tno].pop.num * 1000 / chk,
	   nstats->team[tno].plague.hit,
	   nstats->team[tno].plague.count,
	   nstats->team[tno].plague.low,
	   nstats->team[tno].bomb.num,
	   nstats->team[tno].drop.num,
	   nstats->team[tno].carried,
	   nstats->team[tno].pop.num - nstats->team[tno].plague.count
	   - nstats->team[tno].bomb.num - nstats->team[tno].drop.num
	   - nstats->team[tno].carried);
}

usage ()
{
  fprintf (stderr, "Usage: %s [-FRKOAobl] .inl_timestats [outfile]\n",
	   Prog_Name);

  fprintf (stderr, "       flags are:\n");
  fprintf (stderr, "         FRKO: stats for team Fed Rom Kli Ori\n");
  fprintf (stderr, "               (normally teams are those in combat)\n");
  fprintf (stderr, "         A:    All teams (same as -FRKO)\n");
  fprintf (stderr, "         o:    Old stat files (before INL 4.00)\n");
  fprintf (stderr, "         b:    Byte swap the data (for old stat files)\n");
  fprintf (stderr, "         l:    Long format (incls. player status)\n");

  exit (-1);
}

print_expl (outf)
     FILE *outf;
{
  fprintf (outf, "\n");
  fprintf (outf, "Time is in minutes; stats taken every %g minutes.\n",
	   (double) NSTATS_TICKS_PER / 600.0);
  fprintf (outf, "\n");
  fprintf (outf, "Pops\n");
  fprintf (outf, "  chk:	Number of times planets were checked for a\n");
  fprintf (outf, "  	possible army pop\n");
  fprintf (outf, "  hit:	Number of times a check resulted in a pop\n");
  fprintf (outf, "  num:	Number of armies popped\n");
  fprintf (outf, "  ave:	1000 * chk / num\n");
  fprintf (outf, "\n");
  fprintf (outf, "Plague\n");
  fprintf (outf, "  hit:	Number of plagues that occurred\n");
  fprintf (outf, "  num:	Number of armies lost to plague\n");
  fprintf (outf, "  low:	Lowest remaining army count on any planet\n");
  fprintf (outf, "  	that plagued (-1 indicates no plagues)\n");
  fprintf (outf, "\n");
  fprintf (outf, "Losses\n");
  fprintf (outf, "  bmb:	Armies lost to enemy bombing\n");
  fprintf (outf, "  drp:	Armies lost to enemy dropping\n");
  fprintf (outf, "  car:	Armies lost while on board ship (ogged)\n");
  fprintf (outf, "\n");
  fprintf (outf, "\"net\" is the total gain or loss of armies, or\n");
  fprintf (outf, "     pop num - plague num - bmb - drp - car\n");
  fprintf (outf, "\n");
}

swapb (p, n)
     long *p;
     int n;
{
  int i;

  for (i = 0; i < n; i++, p++)
    *p = ntohl (*p);
}

long_statline (fd, nstats, tno)
     FILE *fd;
     struct st_new_stats *nstats;
     int tno;
{
  int chk;

  if (!doteam[tno])
    return;

  chk = nstats->team[tno].pop.check;
  if (chk < 1)
    chk = 1;

  fprintf (fd,
   "   %3s  %4d %3d %3d %04d  %3d %3d %3d  %3d %3d %3d  %4d  %5d %4d %4d\n",
	   tteamshort[1 << tno - 1],
	   nstats->team[tno].pop.check,
	   nstats->team[tno].pop.hit,
	   nstats->team[tno].pop.num,
	   nstats->team[tno].pop.num * 1000 / chk,
	   nstats->team[tno].plague.hit,
	   nstats->team[tno].plague.count,
	   nstats->team[tno].plague.low,
	   nstats->team[tno].bomb.num,
	   nstats->team[tno].drop.num,
	   nstats->team[tno].carried,
	   nstats->team[tno].pop.num - nstats->team[tno].plague.count
	   - nstats->team[tno].bomb.num - nstats->team[tno].drop.num
	   - nstats->team[tno].carried,
	   nstats->team[tno].player.num,
	   nstats->team[tno].player.lowfuel,
	   nstats->team[tno].player.damaged);
}

header (type)
     char *type;
{
  char buf[100];

  strcpy (buf, type);
  strcat (buf, "/");

  if (lflag)
    long_header (buf);
  else
    short_header (buf);
}

short_header (type)
     char *type;
{
  fprintf (outf, " %-5s      pops             plague       losses\n", type);
  fprintf (outf,
	   " team    chk hit num  ave  hit num low  bmb drp car   net\n");
}

long_header (type)
     char *type;
{
  fprintf (outf,
	   " %-5s      pops             plague       losses             players\n", type);
  fprintf (outf,
	   " team    chk hit num  ave  hit num low  bmb drp car   net  count lowf  dam\n");
}

formaterror (arg)
     char *arg;
{
  if (!arg)
    arg = "unspecified reasons";

  fprintf (stderr, "Input file format error: %s.\n", arg);

  exit (-1);
}
